@@ -6,20 +6,24 @@
 #include "runnermanager.h"
 
 #include <KSharedConfig>
-#include <QAction>
 #include <QCoreApplication>
 #include <QObject>
 #include <QProcess>
 #include <QStandardPaths>
 #include <QTest>
+#include <qtestcase.h>
 
 #include "abstractrunnertest.h"
 #include "kpluginmetadata_utils_p.h"
 
-Q_DECLARE_METATYPE(Plasma::QueryMatch)
-Q_DECLARE_METATYPE(QList<Plasma::QueryMatch>)
+Q_DECLARE_METATYPE(KRunner::QueryMatch)
+Q_DECLARE_METATYPE(QList<KRunner::QueryMatch>)
 
-using namespace Plasma;
+using namespace KRunner;
+namespace KRunner
+{
+extern int __changeCountBeforeSaving;
+}
 
 class RunnerManagerTest : public AbstractRunnerTest
 {
@@ -27,14 +31,12 @@ class RunnerManagerTest : public AbstractRunnerTest
 private Q_SLOTS:
     void initTestCase()
     {
+        __changeCountBeforeSaving = 1;
         startDBusRunnerProcess({QStringLiteral("net.krunnertests.dave")});
         qputenv("XDG_DATA_DIRS", QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation).toLocal8Bit());
         QCoreApplication::setLibraryPaths(QStringList());
         initProperties();
-        auto md = parseMetaDataFromDesktopFile(QFINDTESTDATA("dbusrunnertestmulti.desktop"));
-        QVERIFY(md.isValid());
-        manager->loadRunner(md);
-        qRegisterMetaType<QList<Plasma::QueryMatch>>();
+        qRegisterMetaType<QList<KRunner::QueryMatch>>();
     }
 
     void cleanupTestCase()
@@ -48,8 +50,8 @@ private Q_SLOTS:
      */
     void testScheduleMatchesChanged()
     {
-        QSignalSpy spyQueryFinished(manager.get(), &Plasma::RunnerManager::queryFinished);
-        QSignalSpy spyMatchesChanged(manager.get(), &Plasma::RunnerManager::matchesChanged);
+        QSignalSpy spyQueryFinished(manager.get(), &KRunner::RunnerManager::queryFinished);
+        QSignalSpy spyMatchesChanged(manager.get(), &KRunner::RunnerManager::matchesChanged);
 
         QVERIFY(spyQueryFinished.isValid());
         QVERIFY(spyMatchesChanged.isValid());
@@ -83,9 +85,9 @@ private Q_SLOTS:
         QVERIFY(timer.elapsed() >= 300);
 
         // At this point RunnerManager::jobDone() should have anticipated the final emission.
-        QCOMPARE(spyMatchesChanged.count(), 2); // We had the second matchesChanged emission, now with the query result
         QCOMPARE(manager->matches().count(), 1); // The result is here
         QCOMPARE(spyQueryFinished.count(), 1); // Will have emited queryFinished, job is done
+        QCOMPARE(spyMatchesChanged.count(), 2); // We had the second matchesChanged emission, now with the query result
 
         // Now we will make sure that RunnerManager::scheduleMatchesChanged() emits matchesChanged instantly
         // if we start a query with an empty string. It will never produce results, stalling is meaningless
@@ -101,9 +103,10 @@ private Q_SLOTS:
      */
     void testQueryFinishedFromReset()
     {
-        QSignalSpy spyQueryFinished(manager.get(), &Plasma::RunnerManager::queryFinished);
+        QSignalSpy spyQueryFinished(manager.get(), &KRunner::RunnerManager::queryFinished);
 
         manager->launchQuery("fooDelay1000");
+        QTest::qSleep(500);
         QCOMPARE(spyQueryFinished.size(), 0);
 
         manager->launchQuery("fooDelay300");
@@ -126,6 +129,49 @@ private Q_SLOTS:
 
         manager.launchQuery("somequery");
     }
+
+    void testRunnerManagerStateGroups()
+    {
+        auto stateGrp = KSharedConfig::openConfig(QString(), KConfig::NoGlobals)->group("Testme");
+        auto configGrp = KSharedConfig::openConfig(QString(), KConfig::NoGlobals)->group("Plugins");
+        stateGrp.deleteGroup();
+        RunnerManager manager(configGrp, stateGrp, this);
+        manager.setAllowedRunners({QStringLiteral("fakerunnerplugin")});
+        manager.loadRunner(KPluginMetaData::findPluginById(QStringLiteral("krunnertest"), QStringLiteral("fakerunnerplugin")));
+        QSignalSpy spyQueryFinished(&manager, &KRunner::RunnerManager::queryFinished);
+
+        manager.launchQuery("foo");
+        spyQueryFinished.wait();
+        manager.run(manager.matches().constFirst());
+        manager.matchSessionComplete();
+        QCOMPARE(stateGrp.readEntry("LaunchCounts"), "1 foo");
+        QCOMPARE(stateGrp.config()->groupList().size(), 1);
+    }
+
+    void testRunnerSuspendWhileReloadingConfig()
+    {
+        RunnerManager manager;
+        manager.loadRunner(KPluginMetaData::findPluginById(QStringLiteral("krunnertest2"), QStringLiteral("suspendedrunnerplugin")));
+        QCOMPARE(manager.runners().size(), 1);
+
+        AbstractRunner *runner = manager.runners().constFirst();
+        QVERIFY(runner->isMatchingSuspended());
+
+        QSignalSpy spy(&manager, &KRunner::RunnerManager::queryFinished);
+        manager.launchQuery("foo");
+        QVERIFY2(spy.wait(), "RunnerManager did not emit the queryFinished signal");
+
+        QCOMPARE(manager.matches().size(), 1);
+
+        QVERIFY(!runner->isMatchingSuspended());
+    }
+
+    void testAbstractRunnerTestTimeout()
+    {
+        QEXPECT_FAIL("", "This test is expected to fail", Continue);
+        const auto matches = launchQuery("fooDelay6000");
+        QVERIFY(matches.isEmpty());
+    }
 };
 
 QTEST_MAIN(RunnerManagerTest)
