@@ -1,11 +1,12 @@
 /*
     SPDX-FileCopyrightText: 2006-2007 Aaron Seigo <aseigo@kde.org>
+    SPDX-FileCopyrightText: 2023 Alexander Lohnau <alexander.lohnau@gmx.de>
 
     SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
-#ifndef PLASMA_QUERYMATCH_H
-#define PLASMA_QUERYMATCH_H
+#ifndef KRUNNER_QUERYMATCH_H
+#define KRUNNER_QUERYMATCH_H
 
 #include <QList>
 #include <QSharedDataPointer>
@@ -13,23 +14,19 @@
 
 #include "krunner_export.h"
 
-class QAction;
 class QIcon;
-class QString;
 class QVariant;
-class QWidget;
 
-namespace Plasma
+namespace KRunner
 {
-class RunnerContext;
+class Action;
 class AbstractRunner;
 class QueryMatchPrivate;
 
 /**
  * @class QueryMatch querymatch.h <KRunner/QueryMatch>
  *
- * @short A match returned by an AbstractRunner in response to a given
- * RunnerContext.
+ * @short A match returned by an AbstractRunner in response to a given RunnerContext.
  */
 class KRUNNER_EXPORT QueryMatch
 {
@@ -41,24 +38,13 @@ public:
         NoMatch = 0, /**< Null match */
         CompletionMatch = 10, /**< Possible completion for the data of the query */
         PossibleMatch = 30, /**< Something that may match the query */
-#if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 99)
-        /**
-         * A purely informational, non-runnable match, such as the answer to a question or calculation.
-         * The data of the match will be converted to a string and set in the search field.
-         * @deprecated Since 5.99, call RunnerContext::requestQueryStringUpdate in AbstractRunner::run method instead
-         */
-        InformationalMatch KRUNNER_ENUMERATOR_DEPRECATED_VERSION(5, 99, "Call RunnerContext::requestQueryStringUpdate in AbstractRunner::run method instead") =
-            50,
-#endif
         /**
          * A match that represents an action not directly related
          * to activating the given search term, such as a search
          * in an external tool or a command learning trigger. Helper
          * matches tend to be generic to the query and should not
          * be autoactivated just because the user hits "Enter"
-         * while typing. They must be explicitly selected to
-         * be activated, but unlike InformationalMatch cause
-         * an action to be triggered.
+         * while typing.
          */
         HelperMatch = 70,
         ExactMatch = 100, /**< An exact match to the query */
@@ -81,35 +67,23 @@ public:
     QueryMatch &operator=(const QueryMatch &other);
     bool operator==(const QueryMatch &other) const;
     bool operator!=(const QueryMatch &other) const;
-    bool operator<(const QueryMatch &other) const;
 
     /**
      * @return the runner associated with this action
      */
     AbstractRunner *runner() const;
 
-    /**
-     * Requests this match to activae using the given context
-     *
-     * @param context the context to use in conjunction with this run
-     *
-     * @sa AbstractRunner::run
-     */
-    void run(const RunnerContext &context) const;
-
     /**
      * @return true if the match is valid and can therefore be run,
      *         an invalid match does not have an associated AbstractRunner
      */
     bool isValid() const;
 
-#if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 113)
     /**
      * Sets the type of match this action represents.
+     * This value is used for sorting the different categories
      */
-    KRUNNER_DEPRECATED_VERSION(5, 113, "Use setCategoryRelevance instead")
     void setType(Type type);
-#endif
 
     /**
      * The type of action this is. Defaults to PossibleMatch.
@@ -122,7 +96,7 @@ public:
     enum class CategoryRelevance {
         Lowest = 0,
         Low = 30,
-        Moderate = 51,
+        Moderate = 50,
         High = 70,
         Highest = 100,
     };
@@ -132,16 +106,33 @@ public:
      * This value only affects the sorting of categories and not the sorting within the category. Use @ref setRelevance for this.
      * The value should be from 0 to 100.
      *
-     * @since 5.113
+     * @since 6.0
      */
-    void setCategoryRelevance(CategoryRelevance relevance);
+    void setCategoryRelevance(CategoryRelevance relevance)
+    {
+        setCategoryRelevance(qToUnderlying(relevance));
+    }
 
     /**
-     * Sets information about the type of the match which can
-     * be used to categorize the match.
+     * @internal Internal for now, consumers should utilize CategoryRelevance enum
      *
-     * This string should be translated as it can be displayed
-     * in an UI
+     * @since 6.0
+     */
+    void setCategoryRelevance(qreal relevance);
+
+    /**
+     * Category relevance for this match
+     *
+     * @since 6.0
+     */
+    qreal categoryRelevance() const;
+
+    /**
+     * Sets information about the type of the match which is
+     * used to group the matches.
+     *
+     * This string should be translated as it is displayed in an UI.
+     * The default is @ref AbstractRunner::name
      */
     void setMatchCategory(const QString &category);
 
@@ -149,8 +140,7 @@ public:
      * Extra information about the match which can be used
      * to categorize the type.
      *
-     * By default this returns the internal name of the runner
-     * which returned this result
+     * The default is @ref AbstractRunner::name
      */
     QString matchCategory() const;
 
@@ -171,13 +161,11 @@ public:
     qreal relevance() const;
 
     /**
-     * Sets data to be used internally by the associated
-     * AbstractRunner.
+     * Sets data to be used internally by the runner's @ref AbstractRunner::run implementation.
      *
-     * When set, it is also used to form
-     * part of the id() for this match. If that is inappropriate
-     * as an id, the runner may generate its own id and set that
-     * with setId(const QString&) directly after calling setData
+     * When set, it is also used to form part of the @ref id for this match.
+     * If that is inappropriate as an id, the runner may generate its own
+     * id and set that with @ref setId
      */
     void setData(const QVariant &data);
 
@@ -192,6 +180,13 @@ public:
      * matches from this runner, and should remain constant
      * for the same query for best results.
      *
+     * If the "X-Plasma-Runner-Unique-Results" property from the metadata
+     * is set to true, the runnerId will not be prepended to the ID.
+     * This allows KRunner to de-duplicate results from different runners.
+     * In case the runner's matches are less specific than ones from other runners, the
+     * "X-Plasma-Runner-Weak-Results" property can be set so that duplicates from this
+     * runner are removed.
+     *
      * @param id the new identifying string to use to refer
      *           to this entry
      */
@@ -210,6 +205,7 @@ public:
     /**
      * Sets the main title text for this match; should be short
      * enough to fit nicely on one line in a user interface
+     * For styled and multiline text, @ref setMultiLine should be set to true
      *
      * @param text the text to use as the title
      */
@@ -261,31 +257,14 @@ public:
      */
     QString iconName() const;
 
-#if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 82)
-    /**
-     * Sets the MimeType, if any, associated with this match.
-     * This overrides the MimeType provided by QueryContext, and should only be
-     * set when it is different from the QueryContext MimeType
-     * @deprecated Since 5.82, deprecated for lack of usage
-     */
-    KRUNNER_DEPRECATED_VERSION(5, 82, "deprecated for lack of usage")
-    void setMimeType(const QString &mimeType);
-
-    /**
-     * @return the mimtype for this match, or QString() is none
-     * @deprecated Since 5.82, deprecated for lack of usage
-     */
-    KRUNNER_DEPRECATED_VERSION(5, 82, "deprecated for lack of usage")
-    QString mimeType() const;
-#endif
-
     /**
      * Sets the urls, if any, associated with this match
      */
     void setUrls(const QList<QUrl> &urls);
 
     /**
-     * @return the mimtype for this match, or QString() is none
+     * @return the urls for this match, empty list if none
+     * These will be used in the default implementation of AbstractRunner::mimeDataForMatch
      */
     QList<QUrl> urls() const;
 
@@ -303,35 +282,31 @@ public:
 
     /**
      * Set the actions for this match.
-     * This method allows you to set the actions inside of the AbstractRunner::match() method
-     * and the default implementation of AbstractRunner::actionsForMatch() will return these.
+     * This method allows you to set the actions inside of the AbstractRunner::match method
+     * @see RunnerManager::actionsForMatch
      * @since 5.75
      */
-    void setActions(const QList<QAction *> &actions);
+    void setActions(const QList<KRunner::Action> &actions);
 
     /**
      * Adds an action to this match
      * @since 5.75
      * @see setActions
      */
-    void addAction(QAction *actions);
+    void addAction(const KRunner::Action &action);
 
     /**
      * List of actions set for this match
      * @return actions
      * @since 5.75
      */
-    QList<QAction *> actions() const;
+    QList<KRunner::Action> actions() const;
 
     /**
-     * The current action.
+     * The action that the user has selected when running the match.
+     * This returns a nullptr if no action was selected.
      */
-    QAction *selectedAction() const;
-
-    /**
-     * Sets the selected action
-     */
-    void setSelectedAction(QAction *action);
+    KRunner::Action selectedAction() const;
 
     /**
      * Set if the text should be displayed as a multiLine string
@@ -348,42 +323,13 @@ public:
      */
     bool isMultiLine() const;
 
-#if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 71)
-    /**
-     * @return true if this match can be configured before being run
-     * @since 4.3
-     * @deprecated Since 5.0, this feature has been defunct
-     */
-    KRUNNER_DEPRECATED_VERSION_BELATED(5, 71, 5, 0, "No longer use, feature removed")
-    bool hasConfigurationInterface() const;
-#endif
-
-#if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 71)
-    /**
-     * If hasConfigurationInterface() returns true, this method may be called to get
-     * a widget displaying the options the user can interact with to modify
-     * the behaviour of what happens when the match is run.
-     *
-     * @param widget the parent of the options widgets.
-     * @since 4.3
-     * @deprecated Since 5.0, this feature has been defunct
-     */
-    KRUNNER_DEPRECATED_VERSION_BELATED(5, 71, 5, 0, "No longer use, feature removed")
-    void createConfigurationInterface(QWidget *parent);
-#endif
-
 private:
+    KRUNNER_NO_EXPORT void setSelectedAction(const KRunner::Action &action);
+    friend class RunnerManager;
     QSharedDataPointer<QueryMatchPrivate> d;
 };
 
-}
-
-#if !KRUNNER_ENABLE_DEPRECATED_SINCE(5, 91)
-namespace KRunner
-{
-using QueryMatch = Plasma::QueryMatch;
-using AbstractRunner = Plasma::AbstractRunner;
+/// @since 6.0
+KRUNNER_EXPORT QDebug operator<<(QDebug debug, const KRunner::QueryMatch &match);
 }
 #endif
-
-#endif
